#ifndef LOCAL
   #pragma comment(linker, "/STACK:67108864")
#endif

#include <cstdio>
#include <cctype>
#include <iostream>
#include <fstream>
#include <sstream>
#include <cmath>
#include <cstring>
#include <algorithm>
#include <cstdlib>                                                                                                             
#include <functional>
#include <iterator>
#include <ctime>
#include <cassert>
#include <queue>
#include <stack>
#include <deque>
#include <set>
#include <map>
#include <string>
#include <vector>
#include <utility>

#define MPI 3.141592653589793238462643
#define eps 1e-8
#define inf ((int)1e9)
#define pb push_back
#define mp make_pair
#define zero(a) memset(a,0,sizeof(a))
#define sqr(x) ((x)*(x))

#if defined(__WIN32__) && defined(__GNUC__)
   #define LLD "%I64d"
#else
   #define LLD "%lld"
#endif

#ifndef __GNUC__
#define M_PI 3.1415926535897932384626433832795
#define prev prev_dsfdfgfhgfh
#define next next_mkdlsmfdgkl
#define hash hash_mdfdsgdhgfm
#define move move_njknhkbjkb
#undef assert
#define assert(x) if (!(x)) throw 1
#define sqrt(x) sqrt((double)(x))
#endif

#ifdef LOCAL
  #define NAME __FILE__
#else
  #define NAME "SPbSU #4.Zenit"
#endif

using namespace std;

typedef long long ll;
typedef double ld;

#ifdef LOCAL
	ld START = clock();
	#define TIME (clock()-START)/CLOCKS_PER_SEC
	#define PRINT_TIME cerr << "Time: "<< TIME <<"s"<< endl;
#else
  #define PRINT_TIME (void)0
#endif

#define taskname ""

const int N = 14;
const int M = 20;

string s;

struct World {
  World* tmp;
	int dx[8];
	int dy[8];
  int have[N+10][M+10];
  int d[3][N+10][M+10];
  int prev[3][N+10][M+10];
  int ycoef;
  int curx,cury;
  vector<pair<int,int> > move;
  void addedge(int x,int y,int curx,int cury);
  int parseinput(string s);
  void print();           
  void bfs(int,int,int);
  ld calc(int,int);
  void gen(int,int);
  void go();
  void init();
  void copy(World&);
  bool good(int,int);
  bool border(int,int);
  int getedge(int,int,int,int);
};

World me,sym,symsym;

bool World::good(int x,int y){
	return 0 <= x && x <= N && 0 <= y && y <= M;
}

bool World::border(int x,int y){
	return good(x,y) && (x == 0 || x == N || y == 0 || y == M);
}

int World::getedge(int x,int y,int tx,int ty){
	for (int i = 0; i < 8; i++)
		if (x + dx[i] == tx && y + dy[i] == ty)
			return i;
	return -1;
}

void World::addedge(int x,int y,int curx,int cury){
		int id = getedge(curx,cury,x,y);
		assert(id != -1);
		have[curx][cury] |= (1<<id);

		id = getedge(x,y,curx,cury);
		assert(id != -1);
		have[x][y] |= (1<<id);                
}

int World::parseinput(string s){
	if (s == "Name"){
		printf("%s\n",NAME);
		fflush(stdout);
		return 0;
	}
	if (s == "Quit"){
		exit(0);
	}
	if (s == "Start"){
		ycoef = 1;
		return 1;
	}
	stringstream str(s);
	int n;
	str >> n; 
	--n;

	int x,y;
	str >> x >> y;
	assert(curx == x && cury == ((ycoef == 1)?y:(M-y)));
	for (int i = 0; i < n; i++){
	  str >> x >> y;

    if (ycoef == -1)
    	  y = M - y;

		addedge(curx,cury,x,y);
		curx = x, cury = y;
	}
	return 1;
}
                              
void World::print(){
	assert(move.size());
	printf("%d %d %d",move.size()+1,curx,((ycoef==1)?cury:(M-cury)));
	for (int i = 0; i < (int)move.size(); i++)
		printf(" %d %d",move[i].first,((ycoef==1)?move[i].second:(M-move[i].second)));
	printf("\n");
	fflush(stdout);
	curx = move.back().first;
	cury = move.back().second;
}
                                                
void World::bfs(int type,int x = -1,int y = -1){
	deque<int> qx,qy;
	memset(d[type],63,sizeof(d[type]));
	memset(prev[type],-1,sizeof(prev[type]));
	if (type == 0){
		for (int i = 6; i < 9; i++)
			 qx.pb(i),qy.pb(0),d[type][i][0] = 0;
	}
	if (type == 1){
		for (int i = 6; i < 9; i++)
			 qx.pb(i),qy.pb(M),d[type][i][M] = 0;
	}
	if (type == 2){
	  assert(good(x,y));
		qx.pb(x),qy.pb(y),d[type][x][y] = 0;
	}

	while (!qx.empty()){
		int x = qx.front(), y = qy.front();        
		qx.pop_front(),qy.pop_front();

		if (type == 2 && d[2][x][y] > 0)
			continue;

		if (type == 2 && (y == 0 || y == M) && x >= 6 && x <= 8)
			continue;

		for (int i = 0; i < 8; i++)
			if (!(have[x][y] & (1<<i)))
			{
				int w = (have[x+dx[i]][y+dy[i]] == 0);
				if (d[type][x+dx[i]][y+dy[i]] <= d[type][x][y] + w)
					continue;
				d[type][x+dx[i]][y+dy[i]] = d[type][x][y] + w;
				prev[type][x+dx[i]][y+dy[i]] = i;
				if (w == 0){
				    qx.push_front(x+dx[i]);
				    qy.push_front(y+dy[i]);
				}
				else {
					  qx.push_back(x+dx[i]);
					  qy.push_back(y+dy[i]);
				}
			}	
				
	}

}

ld World::calc(int x,int y){
	if (tmp == 0){
	   return -d[1][x][y] + d[0][x][y];
	}
	assert(this != &symsym);    
  copy(*tmp);              
  tmp->gen(x,M-y); 
  tmp->curx = x, tmp->cury = M-y;
  tmp->bfs(0);
  tmp->bfs(1);
  tmp->bfs(2,x,M-y);
  tmp->move.clear();

  for (int i = 6; i <= 8; i++)
		if (tmp->d[2][i][M] <= 1)
			return -(1<<29); 

	int bx,by;

	bx = by = -1;
	ld res = -(1<<30);

	for (int i = 0; i <= N; i++)
		for (int j = 0; j <= M; j++)
			if (have[i][j] == 0 && tmp->d[2][i][j] == 1){
				ld cur = tmp->calc(i,j);
				if (cur > res || (cur==res && by<=j)){
					res = cur;
					bx = i;
					by = j;
				}
			} 
	return -res;
}

void World::gen(int x,int y){
	assert(good(x,y));
	if (x == curx && y == cury)
 		return;

  addedge(x,y,x-dx[prev[2][x][y]], y - dy[prev[2][x][y]]);
	gen(x-dx[prev[2][x][y]], y - dy[prev[2][x][y]]);

	move.pb(mp(x,y));
}


void World::go(){
	getline(cin,s);
	if (!parseinput(s))
		return;
	move.clear();


	bfs(0);
	bfs(1);
	bfs(2,curx,cury);

  for (int i = 6; i <= 8; i++)
		if (d[2][i][M] <= 1){
			gen(i,M);
			print();
			return;
		}

	int bx,by;

	bx = by = -1;
	ld res = -(1<<30);

	for (int i = 0; i <= N; i++)
		for (int j = 0; j <= M; j++)
			if (have[i][j] == 0 && d[2][i][j] == 1){

				ld cur = calc(i,j);
				if (cur > res || (cur == res && (j >= by))){
					res = cur;
					bx = i;
					by = j;
				}
			} 
	assert(bx != -1);

	gen(bx,by);                 
	print();                       
}

void World::init(){
	ycoef = -1;
	for (int i = 0; i <= N; i++)
		for (int j = 0; j <= M; j++)
			for (int k = 0; k < 8; k++)
				if (!good(i+dx[k],j+dy[k]) || (border(i,j) && (border(i+dx[k],j+dy[k]))))
					have[i][j] |= (1<<k);	
	 curx = N/2;
	 cury = M/2;
}

void World::copy(World& to){  
	 memcpy(to.have,have,sizeof(have));
	 memcpy(to.d,      d,   sizeof(d));
	 memcpy(to.prev,prev,sizeof(prev));

	 for (int i = 0; i <= N; i++)
	 	for (int j = 0; j < M-j; j++){
	 		swap(to.have[i][j],to.have[i][M-j]);
	 	  for (int k = 0; k < 3; k++) {
	 			swap(to.d[k][i][j],to.d[k][i][M-j]);
	 			swap(to.prev[k][i][j],to.prev[k][i][M-j]);
	 	  }
	 	}
	 to.ycoef = -ycoef;
	 to.curx = curx;
	 to.cury = M-cury;
	 to.move = move;
}
                          
int main(){
  const int _dx[8] = {-1, -1, -1,  0, 0,  1, 1, 1};
  const int _dy[8] = {-1,  0,  1, -1, 1, -1, 0, 1};
     
  for (int i = 0; i < 8; i++) {
  	me.dx[i] = _dx[i], me.dy[i] =  _dy[i];
    sym.dx[i] = _dx[i], sym.dy[i] = -_dy[i];
  	symsym.dx[i] = _dx[i], symsym.dy[i] =  _dy[i];

  }

  me.init(); 

  me.tmp = &sym;
  sym.tmp = &symsym;
  symsym.tmp = 0;

  while(true)
  	me.go();

  return 0;
}
