#include <vector>
#include <string>
#include <algorithm>

using namespace std;

#define INF 1000000000

vector<string> SplitByChar(string str, char separator)
{
	vector<string> res;
	int pos = 0;
	int len = 0;
	for (int i = 0; i < str.size(); i++)
	{
		if (str[i] == separator)
		{
			res.push_back(str.substr(pos, len));
			pos = i+1;
			len = 0;
		}
		else len++;
	}
	if (len != 0)
	{
		res.push_back(str.substr(pos, len));
	}
	return res;
}

int ParseInt(string str)
{
	int res = 0;
	for (int i = 0; i < str.size(); i++)
	{
		if (isdigit(str[i]))
			res = res*10 + (str[i] - '0');
	}
	return res;
}

vector<int> Split(string str)
{
	vector<string> args = SplitByChar(str, ' ');
	vector<int> res;
	for (int i = 0; i < args.size(); i++)
	{
		if (args[i].size() != 0)
			res.push_back(ParseInt(args[i]));
	}
	return res;
}

struct Point
{
	int x, y;

	Point(int x = 0, int y = 0)
	{
		this->x = x;
		this->y = y;
	}

	bool operator ==(Point p)
	{
		return x == p.x && y == p.y;
	}
};

const int lines = 14;
const int columns = 20;
const int goalLength = 4;
const int goalStart = ((lines - goalLength) / 2);
const int goalFinish = goalStart + goalLength;



int d[8][2]  = {{-1, -1}, {-1, 0}, {-1, 1}, {0, -1}, {0, 1}, {1, -1}, {1, 0}, {1, 1}};

class Field
{
	bool mark[lines+1][columns+1];
	Point parent[lines+1][columns+1];
	int resx, resy;
	int resDistToMyGoal;
	int resDistToCenter;
	int ourColumn,enemyColumn;
	bool moves[lines+1][columns+1][lines+1][columns+1];
	int x, y;
	vector<Point> myMove;
	vector<Point> final;

	Field(const Field* field)
	{
		memcpy(mark,field->mark,(lines+1)*(columns+1));
		memcpy(moves,field->moves,(lines+1)*(columns+1)*(lines+1)*(columns+1));
		memcpy(parent,field->parent,(lines+1)*(columns+1));		
		resx=field->resx;
		resy=field->resy;
		ourColumn=field->ourColumn;
		x=field->x;
		y=field->y;
	}

	int MovesCount(int x, int y)
	{
		int res = 0;
		for (int i = 0; i <= lines; i++)
			for (int j = 0; j <= columns; j++)
				if (moves[x][y][i][j])
					res++;
		return res;
	}

	void Fill(vector<Point> v)
	{
		for (int i = 1; i < v.size(); i++)
		{
			moves[v[i].x][v[i].y][v[i - 1].x][v[i - 1].y] = true;
			moves[v[i - 1].x][v[i - 1].y][v[i].x][v[i].y] = true;
		}
		x = v[v.size() - 1].x;
		y = v[v.size() - 1].y;
	}

	void InitDfs()
	{
		memset(mark, 0, sizeof(mark));
		memset(parent, 0, sizeof(parent));
		resx = -1;
		resy = -1;
		resDistToMyGoal = columns + 2;
		resDistToCenter = lines + 2;
	}
	


	void Dfs(int curx, int cury, int lastx, int lasty)
	{
		mark[curx][cury] = true;
		parent[curx][cury] = Point(lastx, lasty);
		for (int dir = 0; dir < 8; dir++)
		{
			int dx = d[dir][0];
			int dy = d[dir][1];
			int x = curx + dx;
			int y = cury + dy;
			if (x < 0 || x > lines || y < 0 || y > columns) continue;
			if (!moves[curx][cury][x][y] && !mark[x][y])
			{
				if (MovesCount(x, y) == 0)
				{
					parent[x][y] = Point(curx, cury);
					mark[x][y] = true;
					final.push_back(Point(x,y));
					/*
					int distToMyGoal = abs(y - ourColumn);
					if (distToMyGoal == 0) distToMyGoal = columns + 1;
					int distToCenter = abs(x - lines / 2);
					if (distToMyGoal < resDistToMyGoal || (distToMyGoal == resDistToMyGoal && distToCenter < resDistToCenter))
					{
						resx = x;
						resy = y;
						resDistToMyGoal = distToMyGoal;
						resDistToCenter = distToCenter;
					}
					*/
				}
				else
				{
					Dfs(x, y, curx, cury);
				}
			}
		}
	}

	vector<Point> CollectMove(int x, int y)
	{
		vector<Point> res;
		while (!(x == -1 && y == -1))
		{
			Point p = Point(x, y);
			res.push_back(p);
			x = parent[p.x][p.y].x;
			y = parent[p.x][p.y].y;
		}
		reverse(res.begin(), res.end());
		return res;
	}

	Field* makeMove()
	{
	}

public:
	Field(bool enemyFirst)
	{
		memset(moves,0,(lines+1)*(columns+1)*(lines+1)*(columns+1));
		InitField(enemyFirst);
	}

	void InitField(bool enemyFirst)
	{
		x = lines / 2;
		y = columns / 2;
		ourColumn = enemyFirst ? columns : 0;
		enemyColumn = (!enemyFirst) ? columns : 0;

		for (int i = 0; i <= lines; i++)
		{
			if (i <= goalStart || i >= goalFinish)
			{
				FillEdge(i, 0, i - 1, 0);
				FillEdge(i, columns, i - 1, columns);
				FillEdge(i, 0, i + 1, 0);
				FillEdge(i, columns, i + 1, columns);
			}
		}
		for (int j = 0; j <= columns; j++)
		{
			FillEdge(0, j, 0, j - 1);
			FillEdge(0, j, 0, j + 1);
			FillEdge(lines, j, lines, j - 1);
			FillEdge(lines, j, lines, j + 1);
		}
	}

	void FillEdge(int x1, int y1, int x2, int y2)
	{
		if (x1 < 0 || x1 > lines) return;
		if (x2 < 0 || x2 > lines) return;
		if (y1 < 0 || y1 > columns) return;
		if (y2 < 0 || y2 > columns) return;
		moves[x1][y1][x2][y2] = true;
	}

	vector<Field*> getMoves()
	{
		InitDfs();
		final.clear();
		Dfs(x,y,-1,-1);
		vector<Field*> result;
		for(int i=0;i<final.size();i++)
		{
			Field* f=new Field(*this);
			f->go(final[i]);
			result.push_back(f);
		}
		return result;

	}

	void go(Point p)
	{
		myMove=CollectMove(p.x,p.y);
		Fill(myMove);
	}

	void MakeMove()
	{
		InitDfs();
		Dfs(x, y, -1, -1);
		myMove = CollectMove(resx, resy);
		
		Fill(myMove);
	}

	void printResult()
	{
		
		printf("%d", myMove.size());
		for (int i = 0; i < myMove.size(); i++)
			printf(" %d %d", myMove[i].x, myMove[i].y);
		printf("\n");
	}

	void MakeEnemyMove(string str)
	{
		vector<int> nums = Split(str);
		int moveLen = nums[0];
		vector<Point> points;
		for (int i = 0; i < moveLen; i++)
		{
			int x = nums[i * 2 + 1];
			int y = nums[i * 2 + 2];
			points.push_back(Point(x, y));
		}
		Fill(points);
	}

	int getDistToEnemyGoal(int y)
	{
		return abs(y-enemyColumn);	
	}

	int getDistToEnemyGoal()
	{
		return abs(y-enemyColumn);	
	}

	int getDistToMyGoal(int y)
	{
		return abs(y - ourColumn);
	}

	int getDistToCenter(int x)
	{
		return abs(x - lines / 2);
	}

	int getDistToMyGoal()
	{
		return abs(y - ourColumn);
	}

	int getDistToCenter()
	{
		return abs(x - lines / 2);
	}

};

char buf[1000];

pair<int,int> doIt(Field* field,int deep)
{
	vector<Field*> f=field->getMoves();
	if(f.size()==0)return make_pair(INF,INF);
	int bestG=INF,bestC=INF;
	bool found=false;
	if(!(deep%2))
	{
		bestC=INF;
		bestG=-1;
	}
	for(int i=0;i<f.size();i++)
	{
		if(deep%2)
		{
			if(f[i]->getDistToEnemyGoal()==0)
			{
				found=true;
				bestG=columns;
				bestC=lines/2;
			}
		}
		else
		{
			if(f[i]->getDistToMyGoal()==0)
			{
				found=true;
				bestG=0;
				bestC=lines/2;
			}
		}
	}
	if(!found)
		if(deep==1)
			for(int i=0;i<f.size();i++)
			{
				int g,c;
				c=f[i]->getDistToCenter();
				g=f[i]->getDistToMyGoal();
				//if(deep%2)g=f[i]->getDistToEnemyGoal();else g=f[i]->getDistToMyGoal();
				if(deep%2)
				{
					if (g < bestG || (g == bestG && c < bestC))
					{					
						bestG=g;
						bestC=c;
					}
				}
				else
				{
					if (g > bestG || (g == bestG && c < bestC))
					{
						bestG=g;
						bestC=c;
					}
				}
			
			}
		else
		{
			for(int i=0;i<f.size();i++)
			{
				pair<int,int> cur=doIt(f[i],deep+1);
				int g=cur.first;
				int c=cur.second;
				if(deep%2)
				{
					if (g < bestG || (g == bestG && c < bestC))
					{
						bestG=g;
						bestC=c;
					}
				}
				else
				{
					if (g > bestG || (g == bestG && c < bestC))
					{
						bestG=g;
						bestC=c;
					}
				}
			}
		}
	for(int i=0;i<f.size();i++)delete f[i];
	return make_pair(bestG,bestC);
}

Field* makeMove(Field* field)
{
	vector<Field*> f=field->getMoves();
	if(f.size()==0)return NULL;
	int bestG=-1,bestC=INF;
	Field* result=NULL;
	for(int i=0;i<f.size();i++)
	{
		if(f[i]->getDistToMyGoal()==columns)
		{
			result=f[i];
			break;
		}
	}
	if(result==NULL)
		for(int i=0;i<f.size();i++)
		{
			pair<int,int> cur=doIt(f[i],1);
			int g=cur.first;
			int c=cur.second;
			if (g > bestG || (g == bestG && c < bestC))
			{
				bestG=g;
				bestC=c;
				result=f[i];
			}
		}
	for(int i=0;i<f.size();i++)if(f[i]!=result)delete f[i];
	if(result!=NULL)
	{
		result->printResult();
	}
	return result;
}

void Run() {
	bool firstMove = true;	
	Field* field;
	while (true) {
		gets(buf);
		string str = string(buf);
		if (str == "Quit")
			return;
		if (str == "Name") {
			printf("Arshavin\n");
			fflush(stdout);
			continue;
		}
		if (str != "Start") {
			if (firstMove) {
				field=new Field(true);
				
				firstMove = false;
			}
			field->MakeEnemyMove(str);
		}
		if (firstMove) {
			field=new Field(false);//InitField(false);
			firstMove = false;
		}
		/*
		field->MakeMove();
		field->printResult();
		*/
		field=makeMove(field);
		if(field==NULL)return;
		fflush(stdout);
	}
}

int main()
{
	Run();
}