import static java.lang.Math.abs;
import static java.lang.Math.max;
import static java.lang.Math.min;
import static java.lang.System.exit;
import static java.util.Arrays.fill;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

public class Main {

	static String NAME = "pipebomb";
	static final int WIDTH = 20;
	static final int HEIGHT = 14;

	static boolean invertField;
	static final int[] DX = { 1, 1, 1, 0, -1, -1, -1, 0 };
	static final int[] DY = { 1, 0, -1, -1, -1, 0, 1, 1 };
	static boolean[][] visited = new boolean[HEIGHT + 1][WIDTH + 1];
	static boolean[][][] canNotGo = new boolean[HEIGHT + 1][WIDTH + 1][DX.length];

	static boolean strictlyInsideTheField(int x, int y) {
		return x > 0 && x < HEIGHT && y > 0 && y < WIDTH;
	}

	static boolean insideTheField(int x, int y) {
		return x >= 0 && x <= HEIGHT && y >= 0 && y <= WIDTH;
	}

	static boolean isGate(int x, int y) {
		return x >= 6 && x <= 8 && (y == 0 || y == WIDTH);
	}

	static void initCanGo() {
		for (int i = 0; i <= HEIGHT; i++) {
			for (int j = 0; j <= WIDTH; j++) {
				fill(canNotGo[i][j], false);
				visited[i][j] = false;
				if (strictlyInsideTheField(i, j)) {
					continue;
				} else {
					visited[i][j] = !isGate(i, j);
				}
				for (int dir = 0; dir < DX.length; dir++) {
					int x = i + DX[dir];
					int y = j + DY[dir];
					if (!insideTheField(x, y) || parallel(dir)
							&& !strictlyInsideTheField(x, y)) {
						canNotGo[i][j][dir] = true;
						continue;
					}
				}
			}
		}
		visited[HEIGHT / 2][WIDTH / 2] = true;
	}

	private static boolean parallel(int dir) {
		return abs(DX[dir]) + abs(DY[dir]) == 1;
	}

	static void solve() throws IOException {
		String nameString = nextToken();
		if (!nameString.equals("Name")) {
			throw new AssertionError("noname");
		}
		out.println(NAME);
		invertField = true;
		initCanGo();
		int curX = HEIGHT / 2;
		int curY = WIDTH / 2;
		while (true) {
			String whatWeDoNext = nextToken();
			if (whatWeDoNext.equals("Quit")) {
				break;
			}
			if (!whatWeDoNext.equals("Start")) {
				int n = Integer.parseInt(whatWeDoNext);
				int[] x = new int[n];
				int[] y = new int[n];
				for (int i = 0; i < n; i++) {
					x[i] = nextInt();
					y[i] = nextInt();
				}
				convertMove(x, y);
				processMove(x, y);
				curX = x[n - 1];
				curY = y[n - 1];
			} else {
				invertField = false;
			}
			go(0, curX, curY, LOSE, WIN);
			if (bestX < 0) {
				throw new AssertionError();
			}
			int curVisited[][] = dfsVisited[0];
			List<Integer> bestXX = new ArrayList<Integer>();
			List<Integer> bestYY = new ArrayList<Integer>();
			for (int x = bestX, y = bestY; curVisited[x][y] < 8;) {
				bestXX.add(x);
				bestYY.add(y);
				int d = curVisited[x][y];
				x += DX[d];
				y += DY[d];
			}
			bestXX.add(curX);
			bestYY.add(curY);
			int n = bestXX.size();
			int outX[] = new int[n];
			int outY[] = new int[n];
			for (int i = 0; i < n; i++) {
				outX[i] = bestXX.get(n - i - 1);
				outY[i] = bestYY.get(n - i - 1);
			}
			curX = outX[n - 1];
			curY = outY[n - 1];
			processMove(outX, outY);
			convertMove(outX, outY);
			printMove(outX, outY);
		}
	}

	static void printMove(int x[], int y[]) {
		int n = x.length;
		out.print(n);
		for (int i = 0; i < n; i++) {
			out.print(" " + x[i] + " ");
			out.print(y[i]);
		}
		out.println();
	}

	static int go(int moveNumber, int curX, int curY, int alpha, int beta) {
		if (isGate(curX, curY)) {
			return (curY == 0) ^ (moveNumber % 2 == 0) ? WIN + moveNumber
					: LOSE - moveNumber;
		}
		if (moveNumber == DEPTH) {
			return -getFunction();
		}
		int[][] curVisited = dfsVisited[moveNumber];
		for (int[] e : curVisited) {
			fill(e, -1);
		}
		curVisited[curX][curY] = 8;
		int res = min(dfs(moveNumber, curVisited, curX, curY, alpha, beta),
				LOSE - moveNumber);
		// alpha = min(alpha, res);
		return res;
	}

	static final int DIST_OUR = (getDir(0, 1) - 1) & 7;
	static final int DIST_OPPONENT = (getDir(0, -1) - 1) & 7;

	static int dfs(int moveNumber, int[][] curVisited, int curX, int curY,
			int alpha, int beta) {
		int res = Integer.MAX_VALUE;
		int bestX = -1;
		int bestY = -1;
		int startD = moveNumber % 2 == 0 ? DIST_OUR : DIST_OPPONENT;
		for (int dd = 0; dd < 8; dd++) {
			int d = (startD + dd) & 7;
			if (canNotGo[curX][curY][d]) {
				continue;
			}
			int nextX = curX + DX[d];
			int nextY = curY + DY[d];
			if (curVisited[nextX][nextY] < 0) {
				curVisited[nextX][nextY] = d ^ 4;
				canNotGo[curX][curY][d] = true;
				canNotGo[nextX][nextY][d ^ 4] = true;
				if (visited[nextX][nextY]) {
					int curRes = dfs(moveNumber, curVisited, nextX, nextY,
							alpha, beta);
					if (curRes < res) {
						res = curRes;
						bestX = Main.bestX;
						bestY = Main.bestY;
					}
				} else {
					visited[nextX][nextY] = true;
					int curRes = -go(moveNumber + 1, nextX, nextY, -beta,
							-alpha);
					if (curRes < res) {
						res = curRes;
						bestX = nextX;
						bestY = nextY;
					}
					visited[nextX][nextY] = false;
				}
				canNotGo[curX][curY][d] = false;
				canNotGo[nextX][nextY][d ^ 4] = false;
			}
			alpha = min(alpha, res);
			if (res <= beta) {
				break;
			}
		}
		Main.bestX = bestX;
		Main.bestY = bestY;
		return res;
	}

	static final int DEPTH = 5;
	static int[][][] dfsVisited = new int[DEPTH][HEIGHT + 1][WIDTH + 1];
	static int bestX;
	static int bestY;
	static final int LOSE = 1 << 30;
	static final int WIN = -1 << 30;

	static int getFunction() {
		int distGoal = Integer.MAX_VALUE;
		int distOpponent = Integer.MAX_VALUE;
		int countGoal = 0, countOpponent = 0;
		for (int j = 1; j < WIDTH; j++) {
			if (j > distOpponent) {
				break;
			}
			for (int i = 1; i < HEIGHT; i++) {
				if (!visited[i][j]) {
					continue;
				}
				int d = dist(i, j, 0);
				if (distOpponent > d) {
					distOpponent = d;
					countOpponent = 1;
				} else {
					if (distOpponent == d) {
						countOpponent++;
					}
				}
			}
		}
		for (int j = WIDTH - 1; j > 0; --j) {
			if (WIDTH - j > distGoal) {
				break;
			}
			for (int i = 1; i < HEIGHT; i++) {
				if (!visited[i][j]) {
					continue;
				}
				int d = dist(i, j, WIDTH);
				if (distGoal > d) {
					distGoal = d;
					countGoal = 1;
				} else {
					if (distGoal == d) {
						countGoal++;
					}
				}
				// distOpponent = min(distOpponent, dist(i, j, 0));
			}
		}
		distOpponent = distOpponent * 100 + countOpponent;
		distGoal = distGoal * 100 + countGoal;
		return DEPTH % 2 == 0 ? distOpponent - distGoal : distGoal
				- distOpponent;
	}

	static int dist(int x1, int y1, int y2) {
		int res;
		if (x1 < 6) {
			res = 6 - x1;
		} else if (x1 > 8) {
			res = x1 - 8;
		} else {
			res = 0;
		}
		return max(res, abs(y1 - y2));
	}

	static int getDir(int dx, int dy) {
		for (int i = 0; i < DX.length; i++) {
			if (dx == DX[i] && dy == DY[i]) {
				return i;
			}
		}
		return -1;
	}

	static void convertMove(int[] x, int[] y) {
		int n = x.length;
		if (invertField) {
			for (int i = 0; i < n; i++) {
				y[i] = WIDTH - y[i];
			}
		}
	}

	static void processMove(int[] x, int[] y) {
		int n = x.length;
		for (int i = 0; i + 1 < n; i++) {
			if (!visited[x[i]][y[i]]) {
				throw new AssertionError();
			}
			canNotGo[x[i]][y[i]][getDir(x[i + 1] - x[i], y[i + 1] - y[i])] = true;
			canNotGo[x[i + 1]][y[i + 1]][getDir(x[i] - x[i + 1], y[i]
					- y[i + 1])] = true;
		}
		visited[x[n - 1]][y[n - 1]] = true;
	}

	static void printState() {
		for (int x = 0;; x++) {
			for (int y = 0;; y++) {
				System.err.print(visited[x][y] ? '*' : '.');
				if (y == WIDTH) {
					break;
				}
				if (canNotGo[x][y][7] != canNotGo[x][y + 1][3]) {
					throw new AssertionError();
				}
				System.err.print(canNotGo[x][y][7] ? '-' : ' ');
			}
			System.err.println();
			if (x == HEIGHT) {
				break;
			}
			for (int y = 0;; y++) {
				if (canNotGo[x][y][1] != canNotGo[x + 1][y][5]) {
					throw new AssertionError();
				}
				System.err.print(canNotGo[x][y][1] ? '|' : ' ');
				if (y == WIDTH) {
					break;
				}
				if (canNotGo[x][y][0] != canNotGo[x + 1][y + 1][4]
						|| canNotGo[x + 1][y][6] != canNotGo[x][y + 1][2]) {
					throw new AssertionError();
				}
				System.err
						.print(canNotGo[x][y][0] ? canNotGo[x + 1][y][6] ? 'X'
								: '\\' : canNotGo[x + 1][y][6] ? '/' : ' ');
			}
			System.err.println();
		}
	}

	static BufferedReader br;
	static PrintWriter out;
	static StringTokenizer st;

	public static void main(String[] args) throws Exception {
		br = new BufferedReader(new InputStreamReader(System.in));
		if (System.getProperty("ONLINE_JUDGE") == null) {
			System.setErr(new PrintStream("errors.log"));
			String str = new File("").getAbsolutePath();
			if (str.indexOf('/') >= 0) {
				str = str.substring(str.lastIndexOf('/') + 1);
			}
			if (str.indexOf('\\') >= 0) {
				str = str.substring(str.lastIndexOf('\\') + 1);
			}
			NAME += " " + str;
		}
		out = new PrintWriter(System.out, true);
		try {
			solve();
		} catch (Throwable t) {
			printState();
			t.printStackTrace();
			exit(1);
		}
		out.close();
	}

	static String nextToken() throws IOException {
		while (st == null || !st.hasMoreTokens()) {
			String line = br.readLine();
			if (line == null) {
				return null;
			}
			st = new StringTokenizer(line);
		}
		return st.nextToken();
	}

	static int nextInt() throws IOException {
		return Integer.parseInt(nextToken());
	}

	static long nextLong() throws IOException {
		return Long.parseLong(nextToken());
	}

	static double nextDouble() throws IOException {
		return Double.parseDouble(nextToken());
	}
}
