#include <cassert>
#include <vector>
#include <string>
#include <algorithm>

using namespace std;

FILE* out;
#define eprintf(...) { fprintf(out, __VA_ARGS__); fflush(out); }

vector<string> SplitByChar(string str, char separator)
{
	vector<string> res;
	int pos = 0;
	int len = 0;
	for (int i = 0; i < str.size(); i++)
	{
		if (str[i] == separator)
		{
			res.push_back(str.substr(pos, len));
			pos = i+1;
			len = 0;
		}
		else len++;
	}
	if (len != 0)
	{
		res.push_back(str.substr(pos, len));
	}
	return res;
}

int ParseInt(string str)
{
	int res = 0;
	for (int i = 0; i < str.size(); i++)
	{
		if (isdigit(str[i]))
			res = res*10 + (str[i] - '0');
	}
	return res;
}

vector<int> Split(string str)
{
	vector<string> args = SplitByChar(str, ' ');
	vector<int> res;
	for (int i = 0; i < args.size(); i++)
	{
		if (args[i].size() != 0)
			res.push_back(ParseInt(args[i]));
	}
	return res;
}

struct Point
{
	int x, y;

	Point(int x = 0, int y = 0)
	{
		this->x = x;
		this->y = y;
	}

	bool operator ==(Point p)
	{
		return x == p.x && y == p.y;
	}
};

const int lines = 14;
const int columns = 20;
const int goalLength = 4;
const int goalStart = ((lines - goalLength) / 2);
const int goalFinish = goalStart + goalLength;
int ourColumn;
bool moves[lines+1][columns+1][lines+1][columns+1];
int x, y;


vector<Point> CollectMove(int x, int y, Point parent[lines+1][columns+1]);
double closestEnemyMove(int x, int y);


int MovesCount(int x, int y, bool moves[lines+1][columns+1][lines+1][columns+1])
{
	int res = 0;
	for (int i = 0; i <= lines; i++)
		for (int j = 0; j <= columns; j++)
			if (moves[x][y][i][j])
				res++;
	return res;
}

void Fill(vector<Point> v, bool moves[lines+1][columns+1][lines+1][columns+1])
{
	for (int i = 1; i < v.size(); i++)
	{
		moves[v[i].x][v[i].y][v[i - 1].x][v[i - 1].y] = true;
		moves[v[i - 1].x][v[i - 1].y][v[i].x][v[i].y] = true;
	}
	x = v[v.size() - 1].x;
	y = v[v.size() - 1].y;
}

void MakeEnemyMove(string str)
{
	vector<int> nums = Split(str);
	int moveLen = nums[0];
	vector<Point> points;
	for (int i = 0; i < moveLen; i++)
	{
		int x = nums[i * 2 + 1];
		int y = nums[i * 2 + 2];
		points.push_back(Point(x, y));
	}
	Fill(points, moves);
}

bool mark[lines+1][columns+1];
Point parent[lines+1][columns+1];
int resx, resy;
double resScore;

void InitDfs()
{
	memset(mark, 0, sizeof(mark));
	memset(parent, 0, sizeof(parent));
	resx = -1;
	resy = -1;
	resScore = 1e102;
}

int d[8][2]  = {{-1, -1}, {-1, 0}, {-1, 1}, {0, -1}, {0, 1}, {1, -1}, {1, 0}, {1, 1}};

double edgeScore(int x1, int y1, int x2, int y2) {
	return abs(y2 - ourColumn) - columns / 2;
}





double positionScore(int x, int y, bool enemyDfs) {
	int distToOurGoal = abs(y - ourColumn);
	int distToEnemyGoal = abs(y - columns + ourColumn);
	int distToCenter = abs(x - lines / 2);
	double enemyMove =
		enemyDfs ?
		closestEnemyMove(x, y) :
		0.0;
	double score = enemyMove + distToCenter / 10.0 - distToOurGoal;
	if (y == ourColumn && 6 <= x && x <= 8)
		score = 1e100;
	return score;
}







bool enemyMark[lines+1][columns+1];
bool enemyMoves[lines+1][columns+1][lines+1][columns+1];
Point enemyParent[lines+1][columns+1];
double enemyResScore;

/*
bool counterEnemyMark[lines+1][columns+1];
bool counterEnemyMoves[lines+1][columns+1][lines+1][columns+1];
double counterEnemyResScore;

void dfsCounterEnemy(int curx, int cury, int lastx, int lasty, double dist) {
	counterEnemyMark[curx][cury] = true;
	for (int dir = 0; dir < 8; dir++) {
		int dx = d[dir][0], dy = d[dir][1], x = curx + dx, y = cury + dy;
		if (x < 0 || x > lines || y < 0 || y > columns) continue;
		if (!counterEnemyMoves[curx][cury][x][y] && !counterEnemyMark[x][y]) {
			if (MovesCount(x, y, counterEnemyMoves) == 0) {
				counterEnemyMark[x][y] = true;
				double score = positionScore(x, y, false);
				if (y == columns - ourColumn && 6 <= x && x <= 8)
					score = -1e100;
				if (score < counterEnemyResScore)
					counterEnemyResScore = score;
			} else {
				dfsCounterEnemy(x, y, curx, cury, dist + edgeScore(curx, cury, x, y));
			}
		}
	}
}
double dfsCounterEnemyMove(int x, int y) {
	vector<Point> v = CollectMove(x, y, enemyParent);
	memset(counterEnemyMark, 0, sizeof(counterEnemyMark));
	memcpy(counterEnemyMoves, enemyMoves, sizeof enemyMoves);
	Fill(v, counterEnemyMoves);
	counterEnemyResScore = 1e100;
	dfsCounterEnemy(x, y, -1, -1, 0.0);
	return -counterEnemyResScore;
}
*/

double naiveCounterEnemyMove(int x, int y) {
	double score = 1e100;
	for (int X = 6; X <= 8; X++)
		score = min(score, 1.*max(abs(y - ourColumn), abs(x - X)));
	return score;
}

#define Q 1048576
Point q[Q];
int bfsDist[lines+1][columns+1];

inline int q_next(int i) { return (i + 1) % Q; }
inline int q_prev(int i) { return (i + Q - 1) % Q; }

double bfsCounterEnemyMove(int x, int y) {
	int qb = Q/2, qe = Q/2;
	q[qe++] = Point(x, y);
	for (int X = 0; X <= lines; X++) for (int Y = 0; Y <= columns; Y++)
		bfsDist[X][Y] = 1000000000;
	bfsDist[x][y] = 0;
	while (qb != qe) {
		Point z = q[qb];
		qb = q_next(qb);
		int curx = z.x, cury = z.y;
		for (int dir = 0; dir < 8; dir++) {
			int dx = d[dir][0], dy = d[dir][1];
			x = curx + dx;
			y = cury + dy;
			if (x < 0 || x > lines || y < 0 || y > columns) continue;
			if (enemyMoves[curx][cury][x][y]) continue;
			if (bfsDist[x][y] > bfsDist[curx][cury] + !enemyMark[x][y]) {
				if (enemyMark[x][y]) {
					qb = q_prev(qb);
					q[qb] = Point(x, y);
				} else {
					q[qe] = Point(x, y);
					qe = q_next(qe);
				}
				bfsDist[x][y] = bfsDist[curx][cury] + !enemyMark[x][y];
			}
		}
	}
	double score = 1e100;
	for (int X = 6; X <= 8; X++)
		score = min(score, 1.*bfsDist[X][ourColumn]);
	return score;
}

double closestCounterEnemyMove(int x, int y) {
	return bfsCounterEnemyMove(x, y);
}





void dfsEnemy(int curx, int cury, int lastx, int lasty) {
	enemyMark[curx][cury] = true;
	enemyParent[curx][cury] = Point(lastx, lasty);
	for (int dir = 0; dir < 8; dir++) {
		int dx = d[dir][0], dy = d[dir][1], x = curx + dx, y = cury + dy;
		if (x < 0 || x > lines || y < 0 || y > columns) continue;
		if (!enemyMoves[curx][cury][x][y] && !enemyMark[x][y]) {
			if (MovesCount(x, y, enemyMoves) == 0) {
				enemyMark[x][y] = true;
				enemyParent[x][y] = Point(curx, cury);
				double score = closestCounterEnemyMove(x, y);
				if (y == ourColumn && 6 <= x && x <= 8)
					score = -1e100;
				if (score < enemyResScore) enemyResScore = score;
			} else {
				dfsEnemy(x, y, curx, cury);
			}
		}
	}
}
double closestEnemyMove(int x, int y) {
	vector<Point> v = CollectMove(x, y, parent);
	memset(enemyMark, 0, sizeof(enemyMark));
	memcpy(enemyMoves, moves, sizeof moves);
	Fill(v, enemyMoves);
	enemyResScore = 1e100;
	dfsEnemy(x, y, -1, -1);
	return -enemyResScore;
}

void Dfs(int curx, int cury, int lastx, int lasty, double dist)
{
	mark[curx][cury] = true;
	parent[curx][cury] = Point(lastx, lasty);
	for (int dir = 0; dir < 8; dir++)
	{
		int dx = d[dir][0];
		int dy = d[dir][1];
		int x = curx + dx;
		int y = cury + dy;
		if (x < 0 || x > lines || y < 0 || y > columns) continue;
		if (!moves[curx][cury][x][y] && !mark[x][y])
		{
			if (MovesCount(x, y, moves) == 0)
			{
				parent[x][y] = Point(curx, cury);
				mark[x][y] = true;
				double score = positionScore(x, y, true);
				if (y == columns - ourColumn && 6 <= x && x <= 8) {
					resx = x;
					resy = y;
					score = -1e100;
				}
				if (score < resScore)
				{
					resx = x;
					resy = y;
					resScore = score;
				}
			}
			else
			{
				Dfs(x, y, curx, cury, dist + edgeScore(curx, cury, x, y));
			}
		}
	}
}

vector<Point> CollectMove(int x, int y, Point parent[lines+1][columns+1])
{
	vector<Point> res;
	while (!(x == -1 && y == -1))
	{
		Point p = Point(x, y);
		res.push_back(p);
		x = parent[p.x][p.y].x;
		y = parent[p.x][p.y].y;
	}
	reverse(res.begin(), res.end());
	return res;
}

void MakeMove()
{
	InitDfs();
	Dfs(x, y, -1, -1, 0.0);
	vector<Point> myMove = CollectMove(resx, resy, parent);
	printf("%d", myMove.size());
	for (int i = 0; i < myMove.size(); i++)
		printf(" %d %d", myMove[i].x, myMove[i].y);
	printf("\n");
	Fill(myMove, moves);
}

void FillEdge(int x1, int y1, int x2, int y2)
{
	if (x1 < 0 || x1 > lines) return;
	if (x2 < 0 || x2 > lines) return;
	if (y1 < 0 || y1 > columns) return;
	if (y2 < 0 || y2 > columns) return;
	moves[x1][y1][x2][y2] = true;
}

void InitField(bool enemyFirst)
{
	x = lines / 2;
	y = columns / 2;
	ourColumn = enemyFirst ? columns : 0;

	for (int i = 0; i <= lines; i++)
	{
		if (i <= goalStart || i >= goalFinish)
		{
			FillEdge(i, 0, i - 1, 0);
			FillEdge(i, columns, i - 1, columns);
			FillEdge(i, 0, i + 1, 0);
			FillEdge(i, columns, i + 1, columns);
		}
	}
	for (int j = 0; j <= columns; j++)
	{
		FillEdge(0, j, 0, j - 1);
		FillEdge(0, j, 0, j + 1);
		FillEdge(lines, j, lines, j - 1);
		FillEdge(lines, j, lines, j + 1);
	}
}

char buf[1000];
void Run() {
	bool firstMove = true;	
	while (true) {
		gets(buf);
		string str = string(buf);
		if (str == "Quit")
			return;
		if (str == "Name") {
			printf("cookies_1.6\n");
			fflush(stdout);
			continue;
		}
		if (str != "Start") {
			if (firstMove) {
				InitField(true);
				firstMove = false;
			}
			MakeEnemyMove(str);
		}
		if (firstMove) {
			InitField(false);
			firstMove = false;
		}
		MakeMove();
		fflush(stdout);
	}
}

int main()
{
//	out = fopen("out", "w");
	Run();
//	fclose(out);
}