#include <vector>
#include <iostream>
#include <sstream>
#include <string>
#include <algorithm>
using namespace std;

struct Point
{
	int x, y;

	Point(int x = 0, int y = 0)
	{
		this->x = x;
		this->y = y;
	}

	bool operator ==(Point p)
	{
		return x == p.x && y == p.y;
	}
};

const int INF = 1e9;
const int lines = 15;
const int columns = 21;
const int goalLength = 4;
const int goalStart = ((lines - goalLength) / 2);
const int goalFinish = goalStart + goalLength;
const int DX[] = {1, 1, 0, -1, -1, -1, 0, 1};
const int DY[] = {0, 1, 1, 1, 0, -1, -1, -1};

const int COEF = 3;
const int CUTOFF = 5;

int x = 7, y = 10;
int d[lines][columns];
Point par[lines][columns];
vector<Point> parAns;
int usedEdge[lines][columns][lines][columns], usedPoint[lines][columns];
Point own(7, 20), enemy(7, 0), recoverEnemy;

int dl[lines][columns];

void init()
{
	usedPoint[x][y] = INF;
	for (int i = 0; i < columns; i++)
	{
		if (i)
			usedEdge[0][i - 1][0][i] = usedEdge[0][i][0][i - 1] = INF;
		usedPoint[0][i] = INF;
		if (i)
			usedEdge[lines - 1][i - 1][lines - 1][i] = usedEdge[lines - 1][i][lines - 1][i - 1] = INF;
		usedPoint[lines - 1][i] = INF;
	}
	for (int i = 0; i < lines; i++)
	{
		if (i)
			usedEdge[i - 1][0][i][0] = usedEdge[i][0][i - 1][0] = INF;
		usedPoint[i][0] = INF;
		if (i)
			usedEdge[i - 1][columns - 1][i][columns - 1] = usedEdge[i][columns - 1][i - 1][columns - 1] = INF;
		usedPoint[i][columns - 1] = INF;
	}
}

Point ans;
int best = -INF;

void recover(Point v)
{
	parAns.clear();
	while (v.x != x || v.y != y)
	{
		parAns.push_back(v);
		v = par[v.x][v.y];
	}
	parAns.push_back(v);
}

bool eps (Point a, Point TT)
{
	if (a.y != TT.y)
		return false;
	
	return abs(a.x - TT.x) <= 1;
}

int timer = 1;
bool found;

void dfs(Point v, Point TT)
{
	if (!found && eps(Point(v.x, v.y), TT))
	{
		found = 1;
		recoverEnemy = v;
	}
	if (usedPoint[v.x][v.y] < timer)
		return;
	for (int i = -1; i <= 1; i++)
			for (int j = -1; j <= 1; j++)
				if ((i || j) && v.x + i >= 0 && v.x + i < lines && v.y + j >= 0 && v.y + j < columns)
				{
					if (usedEdge[v.x][v.y][v.x + i][v.y + j] < timer)
					{
						usedEdge[v.x][v.y][v.x + i][v.y + j] = usedEdge[v.x + i][v.y + j][v.x][v.y] = timer;
						par[v.x + i][v.y + j] = v;
						dfs(Point(v.x + i, v.y + j), TT);
					}
				}
}

bool winner(Point v, Point TT)
{
	found = 0;
	dfs(v, TT);
	timer++;
	return found;
}

void path(Point v, int len)
{
	if (usedPoint[v.x][v.y] < INF)
	{
		if (winner(v, own))
			return;
		dl[v.x][v.y] = max(dl[v.x][v.y], len);
		int func1 = COEF * dl[v.x][v.y] - max(abs(v.x - enemy.x), abs(v.y - enemy.y));
		if (func1 > best)
		{
			best = func1;
			ans = v;
			recover(v);
		}
		return;
	}

	usedPoint[v.x][v.y] = INF;
	if (len < CUTOFF)
	{
		for (int i = -1; i <= 1; i++)
			for (int j = -1; j <= 1; j++)
				if ((i || j) && v.x + i >= 0 && v.x + i < lines && v.y + j >= 0 && v.y + j < columns)
					if (usedEdge[v.x][v.y][v.x + i][v.y + j] < INF)
					{
						usedEdge[v.x][v.y][v.x + i][v.y + j] = usedEdge[v.x + i][v.y + j][v.x][v.y] = INF;
						path(Point(v.x + i, v.y + j), len + 1);
						usedEdge[v.x][v.y][v.x + i][v.y + j] = usedEdge[v.x + i][v.y + j][v.x][v.y] = 0;
					}
		usedPoint[v.x][v.y] = 0;
	}
	else
	{
		int rnd[8];
		for (int i = 0; i < 8; i++)
			rnd[i] = i;
		random_shuffle(rnd, rnd + 8);
		for (int i = 0; i < 8; i++)
			if (v.x + DX[i] >= 0 && v.x + DX[i] < lines && v.y + DY[i] >= 0 && v.y + DY[i] < columns)
				if (!usedEdge[v.x][v.y][v.x + DX[i]][v.y + DY[i]] && usedPoint[v.x + DX[i]][v.y + DY[i]])
				{
					usedEdge[v.x][v.y][v.x + DX[i]][v.y + DY[i]] = usedEdge[v.x + DX[i]][v.y + DY[i]][v.x][v.y] = timer;
					path(Point(v.x + DX[i], v.y + DY[i]), len + 1);
					break;
				}
	}
}

void MakeMove()
{
	winner(Point(x, y), enemy);
	if (found)
	{
		recover(recoverEnemy);
		printf("%d ", (int)parAns.size());
		for (int i = (int)parAns.size() - 1; i >= 0; i--)
			printf("%d %d ", parAns[i].x, parAns[i].y);
		printf("\n");
		x = parAns[0].x;
		y = parAns[0].y;
		return;
	}
	path(Point(x, y), 0);
	x = parAns[0].x;
	y = parAns[0].y;
	printf("%d ", (int)parAns.size());
	for (int i = (int)parAns.size() - 1; i >= 0; i--)
		printf("%d %d ", parAns[i].x, parAns[i].y);
	printf("\n");
}

void MakeEnemyMove(string & s)
{
	istringstream ss(s);
	int n;
	ss >> n;
	int ltx = -1, lty = -1;
	for (int i = 0; i < n; i++)
	{
		int tx, ty;
		ss >> tx >> ty;
		usedPoint[tx][ty] = INF;
		if (ltx != -1)
			usedEdge[tx][ty][ltx][lty] = usedEdge[ltx][lty][tx][ty] = INF;
		ltx = tx;
		lty = ty;
		x = tx;
		y = ty;
	}
}

char buf[1000];
void Run() {
	init();
	while (true) {
		gets(buf);
		string str = string(buf);
		if (str == "Quit")
			return;
		if (str == "Name") {
			printf("EpicBot\n");
			fflush(stdout);
			continue;
		}
		if (str == "Start")
			swap(own, enemy);
		else
			MakeEnemyMove(str);
		MakeMove();
		fflush(stdout);
	}
}

int main()
{
	Run();
}