#include <vector>
#include <string>
#include <algorithm>

using namespace std;

vector<string> SplitByChar(string str, char separator)
{
	vector<string> res;
	int pos = 0;
	int len = 0;
	for (int i = 0; i < str.size(); i++)
	{
		if (str[i] == separator)
		{
			res.push_back(str.substr(pos, len));
			pos = i+1;
			len = 0;
		}
		else len++;
	}
	if (len != 0)
	{
		res.push_back(str.substr(pos, len));
	}
	return res;
}

int ParseInt(string str)
{
	int res = 0;
	for (int i = 0; i < str.size(); i++)
	{
		if (isdigit(str[i]))
			res = res*10 + (str[i] - '0');
	}
	return res;
}

vector<int> Split(string str)
{
	vector<string> args = SplitByChar(str, ' ');
	vector<int> res;
	for (int i = 0; i < args.size(); i++)
	{
		if (args[i].size() != 0)
			res.push_back(ParseInt(args[i]));
	}
	return res;
}

struct point
{
	int x, y;

	point(int x = 0, int y = 0)
	{
		this->x = x;
		this->y = y;
	}

	bool operator ==(point p)
	{
		return x == p.x && y == p.y;
	}
	bool operator != (point p) 
	{
		return !(*this == p);
	}

	bool operator < (point p) const
	{
		return false;
	}

};

const int lines = 14;
const int columns = 20;
const int goalLength = 4;
const int goalStart = ((lines - goalLength) / 2);
const int goalFinish = goalStart + goalLength;
int ourColumn;
bool moves[lines+1][columns+1][lines+1][columns+1];
bool rem[lines+1][columns+1][lines+1][columns+1];
bool rem2[lines+1][columns+1][lines+1][columns+1];
int x, y;
int X, Y;

int MovesCount(int x, int y)
{
	int res = 0;
	for (int i = 0; i <= lines; i++)
		for (int j = 0; j <= columns; j++)
			if (moves[x][y][i][j])
				res++;
	return res;
}

void Fill(vector<point> v)
{
	for (int i = 1; i < v.size(); i++)
	{
		moves[v[i].x][v[i].y][v[i - 1].x][v[i - 1].y] = true;
		moves[v[i - 1].x][v[i - 1].y][v[i].x][v[i].y] = true;
	}
	x = v[v.size() - 1].x;
	y = v[v.size() - 1].y;
}

void MakeEnemyMove(string str)
{
	vector<int> nums = Split(str);
	int moveLen = nums[0];
	vector<point> points;
	for (int i = 0; i < moveLen; i++)
	{
		int x = nums[i * 2 + 1];
		int y = nums[i * 2 + 2];
		points.push_back(point(x, y));
	}
	Fill(points);
}

bool mark[lines+1][columns+1];
bool mark2[lines+1][columns+1];
point parent[lines+1][columns+1];
point parent2[lines+1][columns+1];
int resx, resy;
int resDistToMyGoal;
int resDistToCenter;
vector <pair<int, point> > aims;

void InitDfs()
{
	memset(mark, 0, sizeof(mark));
	memset(parent, 0, sizeof(parent));
	resx = -1;
	resy = -1;
	resDistToMyGoal = columns + 2;
	resDistToCenter = lines + 2;
	aims.clear();
}

int d[8][2]  = {{-1, -1}, {-1, 0}, {-1, 1}, {0, -1}, {0, 1}, {1, -1}, {1, 0}, {1, 1}};
int getDeg(point v)
{
	int x = v.x;
	int y = v.y;
	int ans = 0;
	for (int i = 0; i < 8; ++i)
	{
		int nx = x + d[i][0];
		int ny = y + d[i][1];
		if (nx < 0 || ny < 0 || nx > lines || ny > columns) continue;
		if (!moves[x][y][nx][ny] && MovesCount(nx, ny) != 0)
			++ans;
	}
	return ans;
}
void Dfs(int curx, int cury, int lastx, int lasty)
{
	mark[curx][cury] = true;
	parent[curx][cury] = point(lastx, lasty);
	int p[8];
	for (int i = 0; i < 8; ++i)
	{
		p[i] = i;
	}
	random_shuffle(p, p + 8);
	for (int dir = 0; dir < 8; dir++)
	{
		int dx = d[p[dir]][0];
		int dy = d[p[dir]][1];
		int x = curx + dx;
		int y = cury + dy;
		if (x < 0 || x > lines || y < 0 || y > columns) continue;
		if (!moves[curx][cury][x][y] && !mark[x][y])
		{
			if (MovesCount(x, y) == 0)
			{
				parent[x][y] = point(curx, cury);
				mark[x][y] = true;
				int distToMyGoal = abs(y - ourColumn);
				int dist0 = abs(Y - ourColumn);
				if (distToMyGoal == 0) distToMyGoal = columns + 1;
				int distToCenter = abs(x - lines / 2);
				if (distToMyGoal == columns + 1 && distToCenter < 2)
					distToMyGoal = -1000;
				if (getDeg(point(x, y)) == 1 && distToMyGoal < dist0)
					distToMyGoal -= 2;
				aims.push_back(make_pair(distToMyGoal, point(x, y)));
				if (distToMyGoal < resDistToMyGoal || (distToMyGoal == resDistToMyGoal && distToCenter < resDistToCenter))
				{
					resx = x;
					resy = y;
					resDistToMyGoal = distToMyGoal;
					resDistToCenter = distToCenter;
				}
			}
			else
			{
				Dfs(x, y, curx, cury);
			}
		}
	}
}

vector<point> CollectMove(int x, int y)
{
	vector<point> res;
	while (!(x == -1 && y == -1))
	{
		point p = point(x, y);
		res.push_back(p);
		x = parent[p.x][p.y].x;
		y = parent[p.x][p.y].y;
	}
	reverse(res.begin(), res.end());
	return res;
}


point Q[3000];
void delPath(point frm, point to)
{
	memset(mark2, false, sizeof(mark2));
	memset(parent2, false, sizeof(parent2));
	int l = 0;
	int r = 1;
	Q[0] = frm;
	mark2[frm.x][frm.y] = true;
	while (l < r)
	{
		point t = Q[l++];
		for (int i = 0; i < 8; ++i)
		{
			int nx = t.x + d[i][0];
			int ny = t.y + d[i][1];
			if (nx < 0 || ny < 0 || nx > lines || ny > columns) continue;
			if (mark2[nx][ny]) continue;
			if (!moves[t.x][t.y][nx][ny] && MovesCount(nx, ny) != 0)
			{
				mark2[nx][ny] = true;
				Q[r++] = point(nx, ny);
				parent2[nx][ny] = t;
			}
		}
	}
	while(to != frm)
	{
		point prv = parent2[to.x][to.y];
		rem[prv.x][prv.y][to.x][to.y] = !rem[prv.x][prv.y][to.x][to.y];
		rem[to.x][to.y][prv.x][prv.y] = !rem[to.x][to.y][prv.x][prv.y];
		to = prv;
	}
}
vector<point> E;
void findEuler(int x, int y)
{
	for (int i = 0; i < 8; ++i)
	{
		int nx = x + d[i][0];
		int ny = y + d[i][1];
		if (nx < 0 || ny < 0 || nx > lines || ny > columns) continue;
		if (!moves[x][y][nx][ny] && MovesCount(nx, ny) != 0 && !rem[x][y][nx][ny] && !rem2[x][y][nx][ny])
		{
			rem2[x][y][nx][ny] = true;
			rem2[nx][ny][x][y] = true;
			findEuler(nx, ny);
		}
	}
	E.push_back(point(x, y));
}
void doMagic(vector<point> & p) 
{
	memset(rem, false, sizeof(rem));
	memset(rem2, false, sizeof(rem2));
	for (int i = 0; i < p.size(); i += 2)
	{
		delPath(p[i], p[i+1]);
	}
	E.clear();
	findEuler(x, y);
}
vector<point> CollectMove3(int ex, int ey)
{
	point v = parent[ex][ey];
	point st = point(x, y);
	vector<point> p;
	if (v == st)
	{
		if (getDeg(v) % 2 != 0)
			p.push_back(v);
	}
	else
	{
		if (getDeg(v) % 2 == 0)
			p.push_back(v);
		if (getDeg(st) % 2 == 0)
			p.push_back(st);
	}
	for (int i = 0; i <= lines; ++i)
	{
		for (int j = 0; j <= columns; ++j)
		{
			point t = point(i, j);
			if (v == t || t == st) continue;
			if (mark[t.x][t.y] && MovesCount(t.x, t.y) != 0)
				if (getDeg(t) % 2 == 1)
					p.push_back(t);
		}
	}
	doMagic(p);
	reverse(E.begin(), E.end());
	E.push_back(point(ex, ey));
	return E;
}
int mx;
bool used[50][50];
int tx, ty;
void Dfs2(int curx, int cury, int lastx, int lasty)
{
	used[curx][cury] = true;
	for (int dir = 0; dir < 8; dir++)
	{
		int dx = d[dir][0];
		int dy = d[dir][1];
		int x = curx + dx;
		int y = cury + dy;
		if (x < 0 || x > lines || y < 0 || y > columns) continue;
		if (!moves[curx][cury][x][y] && !used[x][y] && (!(rem2[x][y] && !rem[x][y])))
		{
			if (MovesCount(x, y) == 0 && (tx != x || ty != y))
			{
				used[x][y] = true;
				int distToMyGoal = abs(y - ourColumn);
				if (distToMyGoal > mx)
				{
					mx = distToMyGoal;
				}
			}
			else
			{
				Dfs2(x, y, curx, cury);
			}
		}
	}
}
int getVal(int x, int y)
{
	tx = x;
	ty = y;
	mx = 0;
	memset(used, false, sizeof(used));
	Dfs2(x, y, -1, -1);
	return mx;
}
void MakeMove()
{
	InitDfs();
	X = x;
	Y = y;
	Dfs(x, y, -1, -1);
	sort(aims.begin(), aims.end());
	vector<point> myMove;
	int best = 1000;
	for (int i = 0; i < 15 && i < aims.size(); ++i)
	{
		vector<point> mv = CollectMove3(aims[i].second.x, aims[i].second.y);
		int v = getVal(mv.back().x, mv.back().y);
		if (v < best)
		{
			best = v;
			myMove = mv;
		}
	}
	printf("%d", myMove.size());
	for (int i = 0; i < myMove.size(); i++)
		printf(" %d %d", myMove[i].x, myMove[i].y);
	printf("\n");
	Fill(myMove);
}

void FillEdge(int x1, int y1, int x2, int y2)
{
	if (x1 < 0 || x1 > lines) return;
	if (x2 < 0 || x2 > lines) return;
	if (y1 < 0 || y1 > columns) return;
	if (y2 < 0 || y2 > columns) return;
	moves[x1][y1][x2][y2] = true;
}

void InitField(bool enemyFirst)
{
	x = lines / 2;
	y = columns / 2;
	ourColumn = !enemyFirst ? columns : 0;

	for (int i = 0; i <= lines; i++)
	{
		if (i <= goalStart || i >= goalFinish)
		{
			FillEdge(i, 0, i - 1, 0);
			FillEdge(i, columns, i - 1, columns);
			FillEdge(i, 0, i + 1, 0);
			FillEdge(i, columns, i + 1, columns);
		}
	}
	for (int j = 0; j <= columns; j++)
	{
		FillEdge(0, j, 0, j - 1);
		FillEdge(0, j, 0, j + 1);
		FillEdge(lines, j, lines, j - 1);
		FillEdge(lines, j, lines, j + 1);
	}
}

char buf[1000];
void Run() {
	bool firstMove = true;	
	while (true) {
		gets(buf);
		string str = string(buf);
		if (str == "Quit")
			return;
		if (str == "Name") {
			printf("Orange\n");
			fflush(stdout);
			continue;
		}
		if (str != "Start") {
			if (firstMove) {
				InitField(true);
				firstMove = false;
			}
			MakeEnemyMove(str);
		}
		if (firstMove) {
			InitField(false);
			firstMove = false;
		}
		MakeMove();
		fflush(stdout);
	}
}

int main()
{
	srand(146);
	Run();
}