#include <cstdio>
#include <cstring>
#include <cstdlib>
#include <vector>
#include <cmath>
#include <algorithm>

const char name[] = "Euclid2";
const int MAXX = 15;
const int MAXY = 21;
const int MAXN = MAXX * MAXY;

double dist[MAXX][MAXY];
int gamer = -1;

struct Point {
	int x, y;
	Point(int x = 0, int y = 0) : x(x), y(y) {}
	
	void in() {
		scanf("%d%d", &x, &y);
	}

	void out(char eoln) {
		printf("%d %d%c", x, y, eoln);
	}

	bool operator == (const Point & p) const {
		return x == p.x && y == p.y;
	}

	Point operator - (const Point & p) const {
		return Point(x - p.x, y - p.y);
	}

	Point operator * (int k) const {
		return Point(x * k, y * k);
	}

	Point operator + (const Point & p) const {
		return Point(x + p.x, y + p.y);
	}

	double len() {
		return pow(x * x + y * y + 0.1, 0.5);
	}

	bool operator < (const Point & p) const {
		return dist[x][y] * gamer < dist[p.x][p.y] * gamer;
	}
};

typedef std::vector<Point> vp;

Point sh[8] = {
	Point(0, 1),
	Point(1, 1),
	Point(-1, 1),
	Point(1, 0),
	Point(0, -1),
	Point(-1, -1),
	Point(1, -1),
	Point(-1, 0),
};

Point ps[MAXN];
char str[100];

int use[MAXX][MAXY];
int was[MAXX][MAXY];

void inline mark(const Point & st, const Point & p) {
	for (int i = 0; i < 8; i ++) {
		if (p == sh[i])
			use[st.x][st.y] |= 1 << i;
	}
}

vp goset;
int visited[MAXX][MAXY];
int tmpuse[MAXX][MAXY];
int color = 0;

bool correct(const Point & p, int k) {
	if ((use[p.x][p.y] & (1 << k)) || (tmpuse[p.x][p.y] & (1 << k))) 
		return false;
	Point t = p + sh[k];
	return 0 <= t.x && t.x < MAXX && 
				 0 <= t.y && t.y < MAXY &&
				 visited[t.x][t.y] != color;
}

Point q[MAXN];
void makeGoSet(const Point & p) {

	int l = 0;
	int r = 1;
	q[0] = p;
	goset.clear();
	visited[p.x][p.y] = ++ color;
	memset(tmpuse, 0, sizeof(tmpuse));

	while (l < r) {
		Point t = q[l ++];
		for (int i = 0; i < 8; i ++) {
			if (correct(t, i)) {
				Point next = t + sh[i];
				visited[next.x][next.y] = color;
				if (!was[next.x][next.y]) {
					goset.push_back(next);
				}
				else {
					q[r ++] = next;
				}
			}	
		}
	}

	sort(goset.begin(), goset.end());
}

bool findPath(const Point & k, const Point & finish, vp & moves) {
	if (k == finish) {
		moves.clear();
		moves.push_back(k);
		return 1;
	}
	
	for (int i = 0; i < 8; i ++) {
		Point t = k + sh[i];
		if (correct(k, i) && (was[t.x][t.y] || t == finish)) {
			tmpuse[k.x][k.y] |= 1 << i;
			tmpuse[t.x][t.y] |= 1 << (i ^ 4);
			if(findPath(t, finish, moves)) {
				moves.push_back(k);
				return 1;
			}
		}
	}
	return 0;
}


Point readMove() {
	int n;
	if (scanf("%d", &n) == 1) {
		for (int i = 0; i < n; i ++)       
			ps[i].in();
		gets(str);
		was[ps[n - 1].x][ps[n - 1].y] = 1;
		
		for (int i = 1; i < n; i ++) {
			mark(ps[i - 1], ps[i] - ps[i - 1]);
			mark(ps[i], ps[i - 1] - ps[i]);
		}

		makeGoSet(ps[n - 1]);
		
		return ps[n - 1];
	}
	else {
		gets(str);
		if (!strcmp(str, "Quit"))
			exit(0);
		gamer = 1;
		for (int i = 0; i < 8; i ++) {
			sh[i] = sh[i] * (-1);
		}
		makeGoSet(Point(7, 10));
		return Point(7, 10);
	}
}

void init() {
	was[7][10] = 1;
	for (int x = 0; x < MAXX; x ++) {
		for (int y = 0; y < MAXY; y ++) {
			double left = 1000000, right = 1000000;
			Point p = Point(x, y);
			for (int t = 6; t < 9; t ++) {
				left = std::min(left, (p - Point(t, 0)).len());
				right = std::min(right, (p - Point(t, 20)).len());
			}
			dist[x][y] = right - left;
		}
	}
	
	for (int x = 0; x < MAXX; x ++) {
		was[x][0] = 1;
		was[x][MAXY - 1] = 1;
		mark(Point(x, 0), Point(-1, 0));
		mark(Point(x, 0), Point(1, 0));
		mark(Point(x, MAXY - 1), Point(-1, 0));
 		mark(Point(x, MAXY - 1), Point(1, 0));
	}

	for (int y = 0; y < MAXY; y ++) {
		was[0][y] = 1;
		was[MAXX - 1][y] = 1;
		mark(Point(0, y), Point(0, -1));
		mark(Point(0, y), Point(0, 1));
		mark(Point(MAXX - 1, y), Point(0, -1));
		mark(Point(MAXX - 1, y), Point(0, 1));
	}

	for (int x = 6; x < 9; x ++) {
		was[x][0] = 0;
		was[x][MAXY - 1] = 0;
	}
}

vp moves;

void doMove(const Point & p) {
		color ++;
		memset(tmpuse, 0, sizeof(tmpuse));
		findPath(p, goset[0], moves);

		for (int i = 1; i < (int)moves.size(); i ++) {
			mark(moves[i - 1], moves[i] - moves[i - 1]);
			mark(moves[i], moves[i - 1] - moves[i]);
		}
		
		std::reverse(moves.begin(), moves.end());
		was[moves.back().x][moves.back().y] = 1;
		printf("%d ", moves.size());
		for (int i = 0; i < (int) moves.size(); i ++)
			moves[i].out(' ');
		printf("\n");
		fflush(stdout);
}

int main () {
	init();

	gets(str);
	printf("%s\n", name);
	fflush(stdout);
	while (1) {
		doMove(readMove());
	}
}
