#include <iostream>
#include <fstream>
#include <sstream>
#include <string>
#include <vector>
#include <locale>

#include <cstdlib>
#include <ctime>
#include <cassert>
#include <cstring>
#include <cstdint>

namespace mine
{
	const size_t w = 21, h = 15;
	using namespace std;
	struct point
	{
		explicit point(size_t x_=0, size_t y_=0):
			x(x_), y(y_) {}
		bool operator!=(const point &p) const
		{
			return x!=p.x || y!=p.y;
		}
		size_t x, y;
	};
	bool do_invert;
	ostream &operator<<(ostream &out, const point &p)
	{
		if (do_invert)
			return out<<p.y<<' '<<w-p.x-1;
		else
			return out<<p.y<<' '<<p.x;
	}
	istream &operator>>(istream &in, point &p)
	{
		in>>p.y>>p.x;
		if (do_invert)
			p.x = w-p.x-1;
		return in;
	}
	struct player
	{
		uint8_t m_field[w][h];
		vector<point> m_ret;
		player()
		{
			memset(m_field, 0, sizeof(m_field));
		}
		static uint8_t direction(const point &a, const point &b)
		{
			int dx = (int)b.x-(int)a.x;
			int dy = (int)b.y-(int)a.y;
			if (dx<0)
			{
				if (dy<0)
					return 1<<7;
				else if (dy==0)
					return 1<<4;
				else
					return 1<<3;
			}
			else if (dx==0)
			{
				if (dy<0)
					return 1<<6;
				else
				{
					assert(dy>0);
					return 1<<2;
				}
			}
			else
			{
				assert(dx>0);
				if (dy<0)
					return 1<<7;
				else if (dy==0)
					return 1<<0;
				else
					return 1<<1;
			}
		}
		void mark(const vector<point> &v)
		{
			for (size_t i = 1; i<v.size(); ++i)
			{
				m_field[v[i-1].x][v[i-1].y] |= direction(v[i-1], v[i]);
				m_field[v[i].x][v[i].y] |= direction(v[i], v[i-1]);
			}
		}
		const vector<point> &start()
		{
			m_ret.clear();
			m_ret.push_back(point(w/2, h/2));
			m_ret.push_back(point(w/2+1, h/2));
			mark(m_ret);
			return m_ret;
		}
		static inline bool in_field(const point &p)
		{
			return p.x && p.x+1<w && p.y && p.y+1<h;
		}
		void dfs(const point &cur, size_t (&path)[w][h], bool (&used)[w][h], point (&pr)[w][h])
		{
			used[cur.x][cur.y] = true;
			for (int dx = -1; dx<=1; ++dx)
				for (int dy = -1; dy<=1; ++dy)
					if (dx || dy)
					{
						point next(cur.x+dx, cur.y+dy);
						if (!(next.x==0 && 6<=next.y && next.y<=8) && (next.x<w && next.y<h && (in_field(cur) || in_field(next))))
						{
							uint8_t dir = direction(cur, next);
							if (!(m_field[cur.x][cur.y] & dir))
							{
								size_t qual = (rand()%2+1);
								size_t npath = path[cur.x][cur.y]+qual*(!m_field[next.x][next.y])+1;
								if (!used[next.x][next.y] || npath<path[next.x][next.y])
								{
									path[next.x][next.y] = npath;
									pr[next.x][next.y] = cur;
									dfs(next, path, used, pr);
								}
							}
						}
					}
		}
		static void build_path(const point &p, size_t (&path)[w][h], point (&pr)[w][h], point (&nxt)[w][h])
		{
			size_t ps = (size_t)-1;
			point end;
			point end_(20, 0);
			for (int i = 6; i<=8; ++i)
			{
				end_.y = i;
				if (path[end_.x][end_.y]<ps)
				{
					end = end_;
					ps = path[end_.x][end_.y];
				}
			}
			while (p!=end)
			{
				point n = end;
				end = pr[end.x][end.y];
				nxt[end.x][end.y] = n;
			}
		}
		static inline bool in_gate(const point &p)
		{
			return p.x==20 && 6<=p.y && p.y<=8;
		}
		const vector<point> &turn(const vector<point> &v)
		{
			assert(!v.empty());
			mark(v);
			point cur(v.back());
			size_t path[w][h];
			bool used[w][h];
			point pr[w][h];
			point nxt[w][h];
			memset(path, -1, sizeof(path));
			memset(used, 0, sizeof(used));
			dfs(cur, path, used, pr);
			build_path(cur, path, pr, nxt);
			m_ret.clear();
			m_ret.push_back(cur);
			do
			{
				cur = nxt[cur.x][cur.y];
				m_ret.push_back(cur);
			}
			while (!in_gate(cur) && (m_field[cur.x][cur.y] || !in_field(cur)));
			mark(m_ret);
			return m_ret;
		}
		void quit() {}
	};
	struct lexer
	{
		bool m_has_start;
		bool m_valid;
		vector<point> m_v;
		player &m_p;
		lexer(player &p):
			m_has_start(false),
			m_valid(true),
			m_p(p)
		{
			do_invert = false;
		}
		operator bool() const
		{
			return m_valid;
		}
		void iterate()
		{
			string q;
			getline(cin, q);
			const vector<point> *ret = 0;
			if (!q.empty() && isdigit(q[0]))
			{
				if (!m_has_start)
				{
					do_invert = true;
				}
				istringstream sin(q);
				size_t n;
				sin>>n;
				m_v.resize(n);
				for (size_t i = 0; i<n; ++i)
					sin>>m_v[i];
				ret = &m_p.turn(m_v);
			}
			else if (q=="Name")
			{
				cout<<"IzhevskSTU2"<<endl;
			}
			else if (q=="Start")
			{
				m_has_start = true;
				ret = &m_p.start();
			}
			else if (q=="Quit")
			{
				m_p.quit();
				m_valid = false;
			}
			if (ret)
			{
				cout<<ret->size();
				for (size_t i = 0; i<ret->size(); ++i)
					cout<<' '<<ret->at(i);
				cout<<endl;
			}
		}
	};
}

int main()
{
	srand(time(0));
	std::ios_base::sync_with_stdio(false);
#ifdef _DEBUG
	static std::ifstream fin("input.txt");
	static std::ofstream fout("output.txt");
	std::cin.rdbuf(fin.rdbuf());
	std::cout.rdbuf(fout.rdbuf());
#endif
	mine::player p;
	mine::lexer l(p);
	while (l)
		l.iterate();
}
