#include <cstdio>
#include <vector>
#include <algorithm>
#include <iostream>
#include <string>
#include <sstream>
#include <cmath>
#include <stack>
#include <ctime>
#include <cstdlib>

using namespace std;

const int height = 15;
const int width = 21;
const int edges = 2308;

bool areWeLeft = true;

const int shift_column[] = {0, 1, 1, 1};
const int shift_row[] =    {-1, -1, 0, 1};

int our_column, opponent_column;



inline bool onField(int row, int column) {
	return 0 <= row && row < height && 0 <= column && column < width;
}

int head[height * width];
double N[height * width];
double Q[height * width];
int is_border[height * width];
int dest[edges];
int linke[edges];

vector<bool> used_arc(edges >> 1, false);
vector<bool> used_node(height * width, false);


void add_arc(int u, int v) {
	static int ffree = 0;

	used_arc[ffree >> 1] = is_border[u] && is_border[v];

	dest[ffree] = v;
	linke[ffree] = head[u];
	head[u] = ffree++;

	dest[ffree] = u;
	linke[ffree] = head[v];
	head[v] = ffree++;
}

void init_graph() {
	for (int id = 0; id < height * width; ++id) {
		head[id] = -1;
		int row = id / width;
		int column = id % width;
		is_border[id] = (row == 0) || (row == height - 1) || (column == 0) || (column == width - 1);
		used_node[id] = is_border[id];
	}
	used_node[7 * width + 10] = true;
	for (int i = 6; i <= 8; ++i) used_node[i*width + 0] = used_node[i*width + 20] = false;
//	used_node[]
	int total = 0;
	for (int row = 0; row < height; ++row) {
		for (int column = 0; column < width; ++column) {
			for (int shift = 0; shift < 4; ++shift) {
				int to_column = column + shift_column[shift];
				int to_row = row + shift_row[shift];
				if (onField(to_row, to_column)) {
					add_arc(row * width + column, to_row * width + to_column);					
					//total += 2;
				}
			}
		}
	}
	//printf("%d\n", total);
}

vector<bool>* dfs_used_node;
vector<bool>* dfs_used_arc;
int dfs_win_moves, dfs_lose_moves;

int dfs_parent[width * height];
int dfs_visited[width * height];

vector<int>* dfs_leafs;

void dfs1(int u) {
	dfs_visited[u] = true;
	if (!dfs_used_node->at(u)) {
		int row = u / width;
		int column = u % width;
		if (6 <= row && row <= 8 && column == our_column) dfs_lose_moves++;
		else if (6 <= row && row <= 8 && column == opponent_column) dfs_win_moves++;
		else dfs_leafs->push_back(u);
		return;
	}
	for (int arc = head[u]; arc != -1; arc = linke[arc]) {
		if (dfs_used_arc->at(arc >> 1)) continue;
		int v = dest[arc];
		if (!dfs_visited[v]) {
			dfs_parent[v] = arc ^ 1;
			dfs1(dest[arc]);
		}
	}
}

pair<int, int> dfs(vector<bool>& used_node, vector<bool>& used_arc, vector<int>* leafs, int init_id) {
	leafs->resize(0);
	for (int i = 0; i < width * height; ++i) {
		dfs_visited[i] = 0;
		dfs_parent[i] = -1;
	}
	dfs_used_node = &used_node;
	dfs_used_arc = &used_arc;
	dfs_leafs = leafs;
	dfs_win_moves = dfs_lose_moves = 0;
	dfs1(init_id);
	return make_pair(dfs_win_moves, dfs_lose_moves);
}

char buf[1000];

//vector<bool> exused_node;
//vector<bool> exused_edge;
//double exploit(int vertex) {
//	//exused[vertex] = true;
//
//}

vector<bool> fill_node(vector<bool>& used_node, int new_node) {
	vector<bool> new_used_node(used_node);
	new_used_node[new_node] = true;
	return new_used_node;
}

vector<bool> fill_arc(vector<bool>& used_arc, int new_node) {
	vector<bool> new_used_arc(used_arc);
	while (dfs_parent[new_node] != -1) {
		new_used_arc[dfs_parent[new_node] >> 1] = true;
		new_node = dest[dfs_parent[new_node]];
	}
	return new_used_arc;
}

double process_new_vertex(int d, int root, int player, vector<bool>& used_node, vector<bool>& used_arc) {
	used_node[d] = true;
	if (d > 4) {
		int i = root / width;
		int j = root % width;
		double score = (7 - abs(7 - i)) + abs(j - our_column);
		used_node[d] = false;
		return score / (7 + 21);
	}
	vector<int> leafs;
	pair<int, int> result = dfs(used_node, used_arc, &leafs, root);
	
	if (player == 0 && result.first != 0) {
		used_node[d] = false;
		return 1.0; //we win
	}
	if (player == 1 && result.second != 0) return 0.0; //opponent win
	if (leafs.size() == 0) return player == 0 ? 0.0 : 1.0;
	double N = 0.0;
	double Q = 0.0;
	vector<int> order;
	if (leafs.size() < 4) {
		order = leafs;
	}
	else {
		for (int i = 0; i < 4; ++i) {
			order.push_back(leafs[rand() % leafs.size()]);
		}
	}
	for (int i = 0; i < order.size(); ++i) {
		int new_node = order[i];
		vector<bool> new_used_node = fill_node(used_node, new_node); 
		vector<bool> new_used_arc = fill_arc(used_arc, new_node);
		Q += process_new_vertex(d + 1, new_node, 1 - player, new_used_node, new_used_arc);
		N += 1.0;
	}
	used_node[d] = false;
	return Q / N;
}

int main() {
	//freopen("in.txt", "r", stdin);
	srand(31415);
	init_graph();
	for (int i = 0; i < height * width; ++i) {
		N[i] = Q[i] = 0.0;
	}
	int ROW = 7;
	int COLUMN = 10;
	int ID = ROW * width + COLUMN;
	our_column = 20;
	opponent_column = 0;
	while (true) {
		gets(buf);
		string str = string(buf);



		if (str == "Name") {
			printf("SteelAndAlloys\n");
			fflush(stdout);
			continue;
		}

		if (str == "Quit") {
			return 0;
		}








		if (str != "Start") {
			stringstream sstr(str);
			int n;
			sstr >> n;
			int prev_row, prev_column, prev_id;
			sstr >> prev_row >> prev_column;
			prev_id = prev_row * width + prev_column;
			for (int i = 0, arc; i < n - 1; ++i) {
				int row, column, id;
				sstr >> row >> column;
				id = row * width + column;		
				for (arc = head[prev_id]; dest[arc] != id; arc = linke[arc]);
				used_arc[arc >> 1] = true;
				prev_row = row;
				prev_column = column;
				prev_id = id;
			}
			ROW = prev_row;
			COLUMN = prev_column;
		}
		else {
			our_column = 0;
			opponent_column = 20;
			ROW = 7;
			COLUMN = 10;
		}

		ID = ROW * width + COLUMN;
		used_node[ID] = true;
		vector<int> cur_leafs;
		dfs(used_node, used_arc, &cur_leafs, ID);

		if (dfs_win_moves > 0) {
			for (int i = 6; i <= 8; ++i) {
				if (dfs_visited[i * width + opponent_column]) {
					stack<int> winstack;
					int cur = i * width + opponent_column;
					while(cur != ID) {
						winstack.push(cur);
						cur = dest[dfs_parent[cur]];
					}
					winstack.push(ID);
					printf("%d", winstack.size());
					while(!winstack.empty()) {
						printf(" %d %d", winstack.top() / width, winstack.top() % width);
						winstack.pop();
					}
					printf("\n");
					fflush(stdout);
					break;
				}
			}
			continue;
		}
		vector<int> tmp(dfs_parent, dfs_parent + height * width);
		double max_val = -1e10;
		int max_dest = -1;
		for (int i = 0; i < cur_leafs.size(); ++i) {
			int to = cur_leafs[i];
			vector<bool> new_used_arc = fill_node(used_arc, to);
			double cur = process_new_vertex(0, to, 0, used_node, new_used_arc);
			if (max_dest == -1 || cur > max_val) {
				max_val = cur;
				max_dest = to;
			}
		}
		used_node[max_dest] = true;

		stack<int> winstack;
		int i = max_dest / width;
		int j = max_dest % width;
		int cur = i * width + j;
		while(cur != ID) {

			winstack.push(cur);
			used_arc[tmp[cur] >> 1] = true;
			cur = dest[tmp[cur]];
		}
		winstack.push(ID);
		printf("%d", winstack.size());
		while(!winstack.empty()) {
			printf(" %d %d", winstack.top() / width, winstack.top() % width);
			winstack.pop();
		}
		printf("\n");
		fflush(stdout);

	}
	return 0;
}